@@ -42,6 +42,7 @@ if (!missing(thVar)) {
         else
 		z <- thVar
 	z<-embed(z,p+1)[,seq_len(max(thDelay))+1]		#if thDelay=2, ncol(z)=2
+combin<-NULL
 } ###Combination (or single value indicating position) of contemporaneous variables
 else {
 	if (!length(mTh)%in%c(1,k))
@@ -339,45 +340,13 @@ nparbest<-nrow(Bbest)*ncol(Bbest)
 Sigmabest<-matrix(1/t*crossprod(resbest),ncol=k,dimnames=list(colnames(data), colnames(data)))
 SigmabestOls<-Sigmabest*(t/(t-ncol(Bbest)))
 
-###Y and regressors matrix
-tZbest<-t(Zbest)
-naX<-rbind(matrix(NA, ncol=ncol(tZbest), nrow=p), tZbest)
-YnaX<-cbind(data, naX)
-
-
-
-###Pooled AIC
-nobsdown<-sum(dummydown)
-nobsup<-length(dummydown)-nobsdown
-
-SigmabestUnder<-matrix(1/nobsdown*crossprod(resbest*dummydown),ncol=k)
-SigmabestOver<-matrix(1/nobsup*crossprod(resbest*(1-dummydown)),ncol=k)
-
-
-nlikebest<-nobsdown*log(det(SigmabestUnder))+nobsup*log(det(SigmabestOver))
-pooled_aicbest<-nlikebest+2*(nparbest)
-pooled_bicbest<-nlikebest+(log(nobsup)+log(nobsdown))*(nparbest)	#bic #=nlike+log10(t)*4*(1+k); ###BIC
-pooled_info_Thresh<-c(pooled_aicbest, pooled_bicbest)
-
-
-
-
 ###naming and dividing B
 rownames(Bbest) <- paste("Equation", colnames(data))
 LagNames<-c(paste(rep(colnames(data),p), -rep(seq_len(p), each=k)))
-if(include=="const")
-	Bnames<-c("Intercept",LagNames)
-else if(include=="trend")
-	Bnames<-c("Trend",LagNames)
-else if(include=="both")
-	Bnames<-c("Intercept","Trend",LagNames)
-else 
-	Bnames<-c(LagNames)
 
+Bnames<-c(switch(include, const="Intercept", trend="Trend", both=c("Intercept","Trend"), none=NULL),LagNames)
 Blist<-nameB(mat=Bbest, commonInter=commonInter, Bnames=Bnames, nthresh=nthresh, npar=npar)
 BnamesVec<-if(class(Blist)=="list") c(sapply(Blist, colnames)) else colnames(Blist)
-
-colnames(YnaX)<-c(colnames(data),BnamesVec)
 colnames(Bbest)<-BnamesVec
 
 ##number of obs in each regime
@@ -386,6 +355,14 @@ if(nthresh==1)
 else if (nthresh==2)
 	nobs <- c(ndown=ndown, nmiddle=1-nup-ndown,nup=nup)
 
+###Y and regressors matrix
+tZbest<-t(Zbest)
+naX<-rbind(matrix(NA, ncol=ncol(tZbest), nrow=p), tZbest)
+YnaX<-cbind(data, naX)
+BlistMod<-nameB(mat=Bbest, commonInter=commonInter, Bnames=Bnames, nthresh=nthresh, npar=npar,sameName=FALSE )
+BnamesVecMod<-if(class(BlistMod)=="list") c(sapply(BlistMod, colnames)) else colnames(BlistMod)
+colnames(YnaX)<-c(colnames(data),BnamesVecMod)
+
 ###elements to return
 specific<-list()
 specific$allgammas<-allgammas
@@ -434,7 +411,7 @@ summary.TVAR<-function(object,...){
 	k<-x$k
 	t<-x$t
 	p<-x$lag
-	Z<-t(as.matrix(tail(x$model[,-c(1:k)],t)))
+	Z<-t(as.matrix(tail.matrix(x$model[,-c(1:k)],t)))
 	###Stdev, VarCov
 	Sigmabest<-matrix(1/x$t*crossprod(x$residuals),ncol=k)
 	SigmabestOls<-Sigmabest*(x$t/(x$t-ncol(x$coeffmat)))
@@ -624,61 +601,64 @@ toLatex.TVAR<-function(object,..., digits=4, parenthese=c("StDev","Pvalue")){
 }
 
 
-nameB<-function(mat,commonInter, Bnames, nthresh, npar, model=c("TVAR","TVECM"), TVECMmodel="All"){
-	model<-match.arg(model)
-	if(model=="TVAR")
-		sBnames<-Bnames[-which(Bnames=="Intercept")]
-	else if(model=="TVECM")
-		sBnames<-Bnames[-which(Bnames=="ECT")]
-	if(nthresh==1){
-		if(commonInter){
-			if(model=="TVAR")
-				colnames(mat)<-c("Intercept",paste("Dn",sBnames), paste("Up",sBnames))
-			else if(model=="TVECM")
-				colnames(mat)<-c("ECT-","ECT+", sBnames)
-			Blist<-mat}
-		else{
-			colnames(mat) <- rep(Bnames,2)
-			Bdown <- mat[,c(1:npar)]
-			Bup <- mat[,-c(1:npar)]
-			Blist <- list(Bdown=Bdown, Bup=Bup)}
-	}
-	else{
-		if(commonInter){
-			if(model=="TVAR")
-				colnames(mat)<-c("Intercept",paste("Dn",sBnames), paste("Mi",sBnames), paste("Up",sBnames))
-			else if(model=="TVECM")
-				colnames(Bbest)<-c("ECT-","ECT+", sBnames)
-			Blist<-mat}
-		else{
-			colnames(mat)<-rep(Bnames,3)
-			Bdown <- mat[,c(1:npar)]
-			Bmiddle <- mat[,c(1:npar)+npar]
-			Bup <- mat[,c(1:npar)+2*npar]		
-			colnames(Bmiddle) <- Bnames
-			Blist <- list(Bdown=Bdown, Bmiddle=Bmiddle,Bup=Bup)}
-	}
-	return(Blist)
+nameB<-function(mat,commonInter, Bnames, nthresh, npar, model=c("TVAR","TVECM"), TVECMmodel="All", sameName=TRUE){
+  model<-match.arg(model)
+  addRegLetter<-if(sameName) NULL else  c("L ", if(nthresh==1) NULL else "M ", "H ")
+  if(model=="TVAR")
+    sBnames<-Bnames[-which(Bnames=="Intercept")]
+  else if(model=="TVECM")
+    sBnames<-Bnames[-which(Bnames=="ECT")]
+
+##1 threshold
+  if(nthresh==1){
+    if(commonInter){
+      if(model=="TVAR")
+        colnames(mat)<-c("Intercept",paste(rep(addRegLetter, each=length(sBnames)),rep(sBnames,2)),sep="")
+      else if(model=="TVECM")
+        colnames(mat)<-c("ECT-","ECT+", sBnames)
+      Blist<-mat
+    }else{
+      colnames(mat) <- paste(rep(addRegLetter, each=length(Bnames)),rep(Bnames,2),sep="")
+      Bdown <- mat[,c(1:npar)]
+      Bup <- mat[,-c(1:npar)]
+      Blist <- list(Bdown=Bdown, Bup=Bup)}
+##2 thresholds
+  } else{ 
+    if(commonInter){
+      if(model=="TVAR")
+        colnames(mat)<-c("Intercept",paste(rep(addRegLetter, each=length(sBnames)),rep(sBnames,3)),sep="")
+      else if(model=="TVECM")
+        colnames(Bbest)<-c("ECT-","ECT+", sBnames)
+      Blist<-mat}
+    else{
+      colnames(mat)<-paste(rep(addRegLetter, each=length(Bnames)),rep(Bnames,3),sep="")
+      Bdown <- mat[,c(1:npar)]
+      Bmiddle <- mat[,c(1:npar)+npar]
+      Bup <- mat[,c(1:npar)+2*npar]		
+      colnames(Bmiddle) <- Bnames
+      Blist <- list(Bdown=Bdown, Bmiddle=Bmiddle,Bup=Bup)}
+  }
+  return(Blist)
 }
 
 onesearch <- function(thDelay,gammas, fun, trace, gamma, plot){
-	grid1 <- expand.grid(thDelay,gammas)				#grid with delay and gammas
-	store<-mapply(fun,thDelay=grid1[,1],gam1=grid1[,2])
-	posBestThresh <- which(store==min(store, na.rm=TRUE), arr.ind=TRUE)[1]
-
-	if(trace)
-		cat("Best unique threshold", grid1[posBestThresh,2],"\n")
-	if(length(thDelay)>1&trace)
-		cat("Best Delay", grid1[posBestThresh,1],"\n")
-	res<-list(bestThresh=grid1[posBestThresh,2],bestDelay=grid1[posBestThresh,1], allres=cbind(grid1,store))
-        return(res)
+  grid1 <- expand.grid(thDelay,gammas)				#grid with delay and gammas
+  store<-mapply(fun,thDelay=grid1[,1],gam1=grid1[,2])
+  posBestThresh <- which(store==min(store, na.rm=TRUE), arr.ind=TRUE)[1]
+  
+  if(trace)
+    cat("Best unique threshold", grid1[posBestThresh,2],"\n")
+  if(length(thDelay)>1&trace)
+    cat("Best Delay", grid1[posBestThresh,1],"\n")
+  res<-list(bestThresh=grid1[posBestThresh,2],bestDelay=grid1[posBestThresh,1], allres=cbind(grid1,store))
+  return(res)
 }#end of function one search
 
 condiStep<-function(allTh, threshRef, delayRef, fun, trim, trace=TRUE, More=NULL){
   allThUniq <- unique(allTh)
   ng <- length(allTh)
   down<-ceiling(trim*ng)
-
+  
    #correction for case with few unique values
   if(allTh[down]==allTh[down+1]){
     sames<-which(allTh==allTh[down])
